# -*- coding=utf8 -*-
#******************************************************************************
# MediaTypes.py
#------------------------------------------------------------------------------
#
# Copyright (c) 2015 LivingOn <LivingOn@xmail.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#******************************************************************************
import os
from resources.lib.SxxExxKennung import SxxExxKennung
from resources.lib.YoutubePlaylist import YoutubePlaylist

class MediaType(object):
    _PLUGIN = "plugin://plugin.video.youtube/play/?video_id=%s"
    _all_strm_files = []
    _all_strm_folder = []
    
    def create_strm_files(self): 
        raise NotImplemented

    @classmethod
    def activate_all_streams(cls, series_library, movies_library):
        for strmfile in _get_inactive_strms_in(series_library):
            _activate_stream_in(strmfile)
        for strmfile in _get_inactive_strms_in(movies_library):
            _activate_stream_in(strmfile)
        _remove_inactive_file_from(series_library)
        _remove_inactive_file_from(movies_library)

    @classmethod
    def exists_inactive_streams(cls, series_library, movies_library):
        inactive_series = os.path.exists("%sinactive" % series_library)
        inactive_movies = os.path.exists("%sinactive" % movies_library)
        return inactive_series or inactive_movies

    @classmethod
    def all_strm_folder(cls):
        all_folder = []
        [all_folder.append(i) for i in cls._all_strm_folder if not i in all_folder]
        return all_folder

    @classmethod
    def clear_all_strm_folder(cls):
        cls._all_strm_folder = []

class NoMediaFile(MediaType):
    def create_strm_files(self):
        pass

    
class SingleMediaFile(MediaType):
    def __init__(self, librarypath, title, videoid, season=None):
        self._librarypath = librarypath
        self._title = title
        self._videoid = videoid
        self._season = season
        
    def create_strm_files(self):
        folder = _create_strm_folder(self._librarypath, self._title)
        self._all_strm_folder.append(folder)
        title = self._season if self._season else self._title
        strmfile = "%s/%s.strm" % (folder, title)
        if _write_strm_file(strmfile, MediaType._PLUGIN % self._videoid):
            _append_to_inactive_file(strmfile, self._librarypath)
        


class PlaylistFile(MediaType):
    def __init__(self, libraypath, title, playlistid):
        self._librarypath = libraypath
        self._title = title
        self._playlistid = playlistid
        
    def create_strm_files(self):
        folder = _create_strm_folder(self._librarypath, self._title)
        self._all_strm_folder.append(folder)
        for (title, videoid) in YoutubePlaylist.parse(self._playlistid):
            serie = SxxExxKennung.parse(title)
            if serie:
                strmfile = "%s/%s.strm" % (folder, serie)
                if _write_strm_file(strmfile, MediaType._PLUGIN % videoid): 
                    _append_to_inactive_file(strmfile, self._librarypath)
        
                

def _create_strm_folder(librarypath, title):   
    folder = "%s%s" % (librarypath, title)
    try:
        os.mkdir(folder)
    except OSError:
        pass
    return folder    

def _write_strm_file(strmfile, content):
    result = False
    entryline = "%s\n" % content
    if _is_not_in_strm_file(strmfile, entryline):
        entryline = "#%s" % entryline
        try:
            open(strmfile, "a+").write(entryline)
            result = True
        except IOError:
            pass   
    return result

def _append_to_inactive_file(strmfile, librarypath):
    inactive_file = "%sinactive" % librarypath
    entryline = "%s\n" % strmfile
    try:
        open(inactive_file, "a+").write(entryline)
    except IOError:
        pass    
    
def _get_inactive_strms_in(library):
    content = []
    inactive_file = "%sinactive" % library
    try:
        content = open(inactive_file, "rU").readlines()
    except IOError:
        pass
    result = []
    [result.append(i.strip("\n")) for i in content if not i in result]
    return result
    
def _is_not_in_strm_file(strmfile, entryline):
    content = []
    try:
        content = open(strmfile, "rU").readlines()
    except IOError:
        pass    
    return not entryline in content

def _activate_stream_in(strmfile):
    try:
        content = open(strmfile, "rU").readlines()
        new_content = []
        for line in content:
            if line.startswith("#plugin:"):
                new_content.append(line[1:])
            else:
                new_content.append(line)
        open(strmfile, "w").writelines(new_content)
    except IOError:
        pass

def _remove_inactive_file_from(library):
    try:
        os.remove("%sinactive" % library)
    except OSError:
        pass
